@@ -18,7 +18,7 @@
  *  License along with this module; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  *
- * $Id: ncurses_wrap.c,v 1.7 2005/02/26 22:52:36 t-peters Exp $
+ * $Id: ncurses_wrap.c,v 1.11 2005/12/18 20:52:00 t-peters Exp $
  *
  * This file was adapted from the original ncurses header file which
  * has the following copyright statements:
@@ -380,6 +380,21 @@ static VALUE set_ESCDELAY(VALUE dummy, VALUE new_delay)
     return INT2NUM(ESCDELAY);
 }
 #endif
+
+/* This global is wrapper-specific. It denotes the interval after which the
+   terminal is periodically checked for having resized or not. */
+/* time in milliseconds                           */
+static VALUE get_RESIZEDELAY(){return rb_iv_get(mNcurses, "@resize_delay");}
+static VALUE set_RESIZEDELAY(VALUE dummy, VALUE rb_new_delay)
+{
+    int c_new_delay = NUM2INT(rb_new_delay);
+    if (c_new_delay <= 0)
+        rb_raise(rb_eArgError, "delay must be > 0");
+    rb_new_delay = INT2NUM(c_new_delay);
+    rb_iv_set(mNcurses, "@resize_delay", rb_new_delay);
+    return rb_new_delay ;
+}
+
 static
 void
 init_globals_2(void)
@@ -387,6 +402,7 @@ init_globals_2(void)
     rb_iv_set(mNcurses, "@stdscr", Qnil);
     rb_iv_set(mNcurses, "@curscr", Qnil);
     rb_iv_set(mNcurses, "@newscr", Qnil);
+
     rb_define_module_function(mNcurses, "stdscr", 
                               (&get_stdscr), 0);
     rb_define_module_function(mNcurses, "curscr", 
@@ -409,6 +425,12 @@ init_globals_2(void)
     rb_define_module_function(mNcurses, "ESCDELAY=",
                               (&set_ESCDELAY),1);
 #endif
+    /* The maximum delay before screen resize is detected, in milliseconds */
+    rb_iv_set(mNcurses, "@resize_delay", INT2FIX(333));
+    rb_define_module_function(mNcurses, "RESIZEDELAY",
+                              (&get_RESIZEDELAY),0);
+    rb_define_module_function(mNcurses, "RESIZEDELAY=",
+                              (&set_RESIZEDELAY),1);
 }
 #ifdef HAVE_KEYBOUND
 static VALUE rbncurs_keybound(VALUE dummy, VALUE keycode, VALUE count)
@@ -766,6 +788,7 @@ static VALUE rbncurs_getbkgd(VALUE dummy, VALUE arg1) {
 static int rbncurshelper_nonblocking_wgetch(WINDOW *c_win) {
     /* nonblocking wgetch only implemented for Ncurses */
     int halfdelay = NUM2INT(rb_iv_get(mNcurses, "@halfdelay"));
+    int infd = NUM2INT(rb_iv_get(mNcurses, "@infd"));
     double screen_delay = halfdelay * 0.1;
 #ifdef NCURSES_VERSION
     int windelay = c_win->_delay;
@@ -773,25 +796,35 @@ static int rbncurshelper_nonblocking_wgetch(WINDOW *c_win) {
     int windelay = 0;
 #endif
     double window_delay = (windelay >= 0) ? 0.001 * windelay : (1e200*1e200);
+    /* FIXME:                                                  ^ Infinity ^*/
     double delay = (screen_delay > 0) ? screen_delay : window_delay;
     int result;
     struct timeval tv;
     struct timezone tz = {0,0};
     double starttime, nowtime, finishtime;
+    double resize_delay = NUM2INT(get_RESIZEDELAY()) / 1000.0;
+    fd_set in_fds;
     gettimeofday(&tv, &tz);
     starttime = tv.tv_sec + tv.tv_usec * 1e-6;
     finishtime = starttime + delay;
 #ifdef NCURSES_VERSION
     c_win->_delay = 0;
 #endif
-    while ((result = wgetch(c_win)) == ERR && delay > 0) {
-        tv.tv_sec = 0;
-        tv.tv_usec =  (unsigned)( ( (0.1 < delay) ? 0.1 : delay) * 1e6 );
-        rb_thread_wait_for(tv);
-
+    while (doupdate() /* detects resize */, (result = wgetch(c_win)) == ERR) {
         gettimeofday(&tv, &tz);
         nowtime = tv.tv_sec + tv.tv_usec * 1e-6;
         delay = finishtime - nowtime;
+	if (delay <= 0) break;
+
+	/* Check for terminal size change every resize_delay seconds */
+        if (resize_delay > delay) resize_delay = delay;
+        tv.tv_sec = (time_t)resize_delay;
+        tv.tv_usec = (unsigned)( (resize_delay - tv.tv_sec) * 1e6 );
+
+	/* sleep on infd until input is available or tv reaches timeout */
+	FD_ZERO(&in_fds);
+	FD_SET(infd, &in_fds);
+	rb_thread_select(infd + 1, &in_fds, NULL, NULL, &tv);
     }
 #ifdef NCURSES_VERSION
     c_win->_delay = windelay;
@@ -933,6 +966,9 @@ static VALUE rbncurs_initscr(VALUE dummy) {
     rb_define_const(mNcurses, "ACS_SSSS",      INT2NUM(ACS_SSSS));
 #endif
     rbncurshelper_halfdelay_cbreak_restore();
+    rb_iv_set(mNcurses, "@infd", INT2FIX(0));
+    rb_iv_set(mNcurses, "@halfdelay", INT2FIX(0));
+    rb_iv_set(mNcurses, "@cbreak", Qfalse);
     return v;
 }
 static VALUE rbncurs_init_color(VALUE dummy, VALUE arg1, VALUE arg2, VALUE arg3, VALUE arg4) {
@@ -1241,8 +1277,15 @@ static VALUE rbncurs_scr_set(VALUE dummy, VALUE arg1) {
 static VALUE rbncurs_setscrreg(VALUE dummy, VALUE arg1, VALUE arg2) {
     return INT2NUM(setscrreg(NUM2INT(arg1),  NUM2INT(arg2)));
 }
-static VALUE rbncurs_set_term(VALUE dummy, VALUE arg1) {
-    VALUE rb_old_screen = wrap_screen(set_term(get_screen(arg1)));
+static VALUE rbncurs_set_term(VALUE dummy, VALUE rb_new_screen) {
+    VALUE rb_old_screen = wrap_screen(set_term(get_screen(rb_new_screen)));
+    rb_iv_set(rb_old_screen, "@infd",      rb_iv_get(mNcurses, "@infd"));
+    rb_iv_set(rb_old_screen, "@halfdelay", rb_iv_get(mNcurses, "@halfdelay"));
+    rb_iv_set(rb_old_screen, "@cbreak",    rb_iv_get(mNcurses, "@cbreak"));
+    rb_iv_set(mNcurses, "@infd",      rb_iv_get(rb_new_screen, "@infd"));
+    rb_iv_set(mNcurses, "@halfdelay", rb_iv_get(rb_new_screen, "@halfdelay"));
+    rb_iv_set(mNcurses, "@cbreak",    rb_iv_get(rb_new_screen, "@cbreak"));
+
     rbncurshelper_halfdelay_cbreak_restore();
     return rb_old_screen;
 }
@@ -1561,6 +1604,12 @@ static VALUE rbncurs_newterm(VALUE dummy, VALUE rb_type, VALUE rb_outfd, VALUE r
         Init_ncurses_full();
         rbncurshelper_halfdelay_cbreak_restore();
     }
+    rb_iv_set(mNcurses, "@infd", INT2NUM(infd));
+    rb_iv_set(rb_screen, "@infd", INT2NUM(infd));
+    rb_iv_set(mNcurses, "@halfdelay", INT2FIX(0));
+    rb_iv_set(rb_screen, "@halfdelay", INT2FIX(0));
+    rb_iv_set(mNcurses, "@cbreak", Qfalse);
+    rb_iv_set(rb_screen, "@cbreak", Qfalse);
     return rb_screen;
 }
 
@@ -2387,7 +2436,7 @@ static void init_functions_3(void)
     NCFUNC(ungetmouse, 1);
 #endif
 #ifdef HAVE_MOUSEMASK
-    NCFUNC(mousemask, 1);
+    NCFUNC(mousemask, 2);
 #endif
 #ifdef HAVE_WENCLOSE
     rb_define_module_function(mNcurses, "wenclose?",
@@ -2523,10 +2572,10 @@ static void init_constants_4(void)
 #define rb_ACS(ACS)                               \
 VALUE rb_## ACS (VALUE rb_screen)                 \
 {                                                 \
-    SCREEN * c_screen = get_screen(rb_screen);    \
-    SCREEN * current_screen = set_term(c_screen); \
+    VALUE current_screen =                        \
+      rbncurs_set_term(mNcurses, rb_screen);      \
     VALUE rb_ACS_CONST = INT2NUM(ACS);            \
-    set_term(current_screen);                     \
+    rbncurs_set_term(mNcurses, current_screen);   \
     return  rb_ACS_CONST;                         \
 }             
 #define wrap_ACS(ACS)                                          \
@@ -2648,8 +2697,14 @@ void Init_ncurses_bin(void)
     eNcurses = rb_define_class_under(mNcurses, "Exception", rb_eRuntimeError);
     rb_iv_set(mNcurses, "@windows_hash", rb_hash_new());
     rb_iv_set(mNcurses, "@screens_hash", rb_hash_new());
-    rb_iv_set(mNcurses, "@halfdelay", INT2NUM(0));
+
+    /* keep track of "halfdelay" settings in the wrapper */
+    rb_iv_set(mNcurses, "@halfdelay", INT2FIX(0));
     rb_iv_set(mNcurses, "@cbreak", Qfalse);
+
+    /* filedescriptor that transports input from terminal to application */
+    rb_iv_set(mNcurses, "@infd", Qnil);
+
     cWINDOW  = rb_define_class_under(mNcurses, "WINDOW", rb_cObject);
     cSCREEN  = rb_define_class_under(mNcurses, "SCREEN", rb_cObject);
     init_constants_1();
